<?php
/**
 * CommentGuestbook Frontend Filter Class
 *
 * @package comment-guestbook
 */

// declare( strict_types=1 ); Remove for now due to warnings in php <7.0!
if ( ! defined( 'WPINC' ) ) {
	exit();
}

require_once CGB_PATH . 'includes/options.php';

/**
 * CommentGuestbook Frontend Filter Class
 *
 * This class handles all filters to overwrite the defaults according to the CommentGuestbook settings.
 */
class CGB_Filters {

	/**
	 * Options class instance reference
	 *
	 * @var CGB_Options
	 */
	private $options;

	/**
	 * Holds the variable where the filter setting was called from
	 *
	 * @var string
	 */
	private $called_from;


	/**
	 * Class constructor which initializes required variables
	 *
	 * @param string $called_from  If the function was called from 'shortcode' or 'after_new_comment'.
	 * @return void
	 */
	public function __construct( $called_from = 'shortcode' ) {
		$this->options     = &CGB_Options::get_instance();
		$this->called_from = $called_from;
		$this->prepare_filters();
	}


	/**
	 * Prepare the required filters
	 *
	 * @return void
	 */
	public function prepare_filters() {
		add_filter( 'comments_open', array( &$this, 'filter_comments_open' ), 50, 2 );
		add_filter( 'option_comment_registration', array( &$this, 'filter_ignore_comment_registration' ) );
		add_filter( 'option_comment_moderation', array( &$this, 'filter_ignore_comment_moderation' ) );
		add_filter( 'option_require_name_email', array( &$this, 'filter_require_name_email' ) );
	}


	/**
	 * Filter to override comments_open status.
	 *
	 * @param bool $open    Whether the current post is open for comments.
	 * @param int  $post_id The post ID.
	 * @return bool
	 */
	public function filter_comments_open( $open, $post_id ) {
		if ( ! $open && (bool) $this->options->get( 'cgb_ignore_comments_open' ) ) {
			return true;
		}
		return $open;
	}


	/**
	 * Filter to override registration requirements for comments
	 *
	 * @param bool $option_value The actual value of the option "comment_registration".
	 * @return bool
	 */
	public function filter_ignore_comment_registration( $option_value ) {
		if ( $this->options->get( 'cgb_ignore_comment_registration' ) ) {
			return false;
		}
		return $option_value;
	}


	/**
	 * Filter to override moderation requirements for comments on guestbook page
	 *
	 * @param bool $option_value The actual value of the option "comment_moderation".
	 * @return bool
	 */
	public function filter_ignore_comment_moderation( $option_value ) {
		if ( $this->options->get( 'cgb_ignore_comment_moderation' ) ) {
			return false;
		}
		return $option_value;
	}


	/**
	 * Filter to override email requirement for a new comment if the email field is removed.
	 *
	 * @param string $option_value The actual value of the option "require_name_email".
	 * @return string
	 */
	public function filter_require_name_email( $option_value ) {
		// Check if the given wp-option is enabled.
		// Use the given default value.
		if ( empty( $option_value ) ) {
			return $option_value;
		}
		// Check if the "require name, email" option is disabled for comment-guestbook comments.
		if ( (bool) $this->options->get( 'cgb_form_require_no_name_mail' ) ) {
			return '';
		}
		// Check if the plugin options require an override.
		if ( (bool) $this->options->get( 'cgb_form_remove_mail' ) || (bool) $this->options->get( 'cgb_page_remove_mail' ) ) {
			$user = wp_get_current_user();
			// Check if the user is logged in and if a valid author name is given.
			// phpcs:ignore WordPress.Security.NonceVerification.Missing
			$author = isset( $_POST['author'] ) ? sanitize_title( stripslashes_deep( $_POST['author'] ) ) : '';
			if ( ! $user->exists() && ! empty( $author ) ) {
				// Override value.
				return '';
			}
		}
		return $option_value;
	}

}
